# Generated by Django 2.0.4 on 2018-04-10 11:59

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('spacyal', '0003_auto_20180410_1115'),
    ]

    operations = [
        migrations.AddField(
            model_name='al_project',
            name='users_allowed',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL),
        ),
    ]
