# Generated by Django 2.0.4 on 2018-04-10 15:08

from django.db import migrations, models
import spacyal.models


class Migration(migrations.Migration):

    dependencies = [
        ('spacyal', '0004_al_project_users_allowed'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='al_project',
            name='data_path',
        ),
        migrations.AddField(
            model_name='al_project',
            name='model',
            field=models.FileField(blank=True, null=True, upload_to=spacyal.models.get_filepath_model),
        ),
        migrations.AddField(
            model_name='al_project',
            name='texts',
            field=models.FileField(default='', upload_to=spacyal.models.get_filepath_texts),
            preserve_default=False,
        ),
    ]
